@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -27,6 +27,44 @@
 
 #include "connman.h"
 
+enum connman_policy {
+	CONNMAN_POLICY_UNKNOWN  = 0,
+	CONNMAN_POLICY_SINGLE   = 1,
+	CONNMAN_POLICY_MULTIPLE = 2,
+	CONNMAN_POLICY_ASK      = 3,
+};
+
+static enum connman_policy global_policy = CONNMAN_POLICY_SINGLE;
+static connman_bool_t global_offlinemode = FALSE;
+
+static const char *policy2string(enum connman_policy policy)
+{
+	switch (policy) {
+	case CONNMAN_POLICY_UNKNOWN:
+		break;
+	case CONNMAN_POLICY_SINGLE:
+		return "single";
+	case CONNMAN_POLICY_MULTIPLE:
+		return "multiple";
+	case CONNMAN_POLICY_ASK:
+		return "ask";
+	}
+
+	return NULL;
+}
+
+static enum connman_policy string2policy(const char *policy)
+{
+	if (g_str_equal(policy, "single") == TRUE)
+		return CONNMAN_POLICY_SINGLE;
+	else if (g_str_equal(policy, "multiple") == TRUE)
+		return CONNMAN_POLICY_MULTIPLE;
+	else if (g_str_equal(policy, "ask") == TRUE)
+		return CONNMAN_POLICY_ASK;
+	else
+		return CONNMAN_POLICY_UNKNOWN;
+}
+
 static void append_profiles(DBusMessageIter *dict)
 {
 	DBusMessageIter entry, value, iter;
@@ -105,34 +143,19 @@ static void append_connections(DBusMessageIter *dict)
 	dbus_message_iter_close_container(dict, &entry);
 }
 
-static void append_state(DBusMessageIter *dict, const char *state)
-{
-	DBusMessageIter entry, value;
-	const char *key = "State";
-
-	dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY,
-								NULL, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_STRING_AS_STRING, &value);
-
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_STRING, &state);
-
-	dbus_message_iter_close_container(&entry, &value);
-
-	dbus_message_iter_close_container(dict, &entry);
-}
-
 static DBusMessage *get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
+	const char *str;
 
 	DBG("conn %p", conn);
 
+	if (__connman_security_check_privilege(msg,
+					CONNMAN_SECURITY_PRIVILEGE_PUBLIC) < 0)
+		return __connman_error_permission_denied(msg);
+
 	reply = dbus_message_new_method_return(msg);
 	if (reply == NULL)
 		return NULL;
@@ -150,15 +173,71 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	append_connections(&dict);
 
 	if (__connman_element_count(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION) > 0)
-		append_state(&dict, "online");
+		str = "online";
 	else
-		append_state(&dict, "offline");
+		str = "offline";
+
+	connman_dbus_dict_append_variant(&dict, "State",
+						DBUS_TYPE_STRING, &str);
+
+	str = policy2string(global_policy);
+	if (str != NULL)
+		connman_dbus_dict_append_variant(&dict, "Policy",
+						DBUS_TYPE_STRING, &str);
+
+	connman_dbus_dict_append_variant(&dict, "OfflineMode",
+				DBUS_TYPE_BOOLEAN, &global_offlinemode);
 
 	dbus_message_iter_close_container(&array, &dict);
 
 	return reply;
 }
 
+static DBusMessage *set_property(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	DBusMessageIter iter, value;
+	const char *name;
+
+	DBG("conn %p", conn);
+
+	if (dbus_message_iter_init(msg, &iter) == FALSE)
+		return __connman_error_invalid_arguments(msg);
+
+	dbus_message_iter_get_basic(&iter, &name);
+	dbus_message_iter_next(&iter);
+	dbus_message_iter_recurse(&iter, &value);
+
+	if (__connman_security_check_privilege(msg,
+					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
+		return __connman_error_permission_denied(msg);
+
+	if (g_str_equal(name, "Policy") == TRUE) {
+		enum connman_policy policy;
+		const char *str;
+
+		dbus_message_iter_get_basic(&value, &str);
+		policy = string2policy(str);
+		if (policy == CONNMAN_POLICY_UNKNOWN)
+			return __connman_error_invalid_arguments(msg);
+
+		global_policy = policy;
+	} else if (g_str_equal(name, "OfflineMode") == TRUE) {
+		connman_bool_t offlinemode;
+
+		dbus_message_iter_get_basic(&value, &offlinemode);
+
+		if (global_offlinemode == offlinemode)
+			return __connman_error_invalid_arguments(msg);
+
+		global_offlinemode = offlinemode;
+
+		__connman_device_set_offlinemode(offlinemode);
+	}
+
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+}
+
 static DBusMessage *register_agent(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -208,9 +287,10 @@ static DBusMessage *unregister_agent(DBusConnection *conn,
 }
 
 static GDBusMethodTable manager_methods[] = {
-	{ "GetProperties",   "",  "a{sv}", get_properties   },
-	{ "RegisterAgent",   "o", "",      register_agent   },
-	{ "UnregisterAgent", "o", "",      unregister_agent },
+	{ "GetProperties",   "",   "a{sv}", get_properties   },
+	{ "SetProperty",     "sv", "",      set_property     },
+	{ "RegisterAgent",   "o",  "",      register_agent   },
+	{ "UnregisterAgent", "o",  "",      unregister_agent },
 	{ },
 };
 
